<?php
/**
 * aLinks Deeper Blog Keyphrase Module
 *
 * A keyphrase module that creates links to other post within your blog.
 *
 * @author	Sean Hickey <seanhickey@gmail.com>
 * @license	GPL
 * @version	$Id: mod.php 46 2008-03-24 18:02:58Z sean $
 */
 
/**
 * You can adjust the following settings to tweak the modules search algorithm.
 * DEEPER_BLOG_FOUND_TITLE : The number of points given if the keyphrase is found in a post's title
 * DEEPER_BLOG_FOUND_EXCERPT : The number of points given if the keyphrase is found in a post's excerpt
 * DEEPER_BLOG_FOUND_CONTENT : The number of points given if the keyphrase is found in a post's content
 * DEEPER_BLOG_SENSITIVITY  : All the points added up must be greater than or equal to this number for a post
 * to be considered a "hit".
 */
define('DEEPER_BLOG_FOUND_TITLE', 5);
define('DEEPER_BLOG_FOUND_EXCERPT', 3);
define('DEEPER_BLOG_FOUND_CONTENT', 1);
define('DEEPER_BLOG_SENSITIVITY', 5);

class DeeperBlogKeyphrases extends aLinksKeyphrases {

	/**
	 * The safe name for this module.
	 * @var string
	 */
	var $name = 'deeper_blog';
	
	/**
	 * Are FULLTEXT indexes turned on for the post_title,
	 * post_content, and post_excerpt columns.
	 * @var bool
	 */
	var $fullText = false;
	
	/**
	 * Has the module already checked to see if FULLTEXT
	 * indexes are turned on for the columns.
	 * @var bool
	 */
	var $fullTextChecked = false;
	
	function doPhrase($postObj, $keyphrase, $description, $defines) {
		$this->isFullText();
		if (empty($this->options['days'])) {
			$maxDays = 20;
		} else {
			$maxDays = $this->options['days'];
		}
		$postUrl = $this->findPosts($postObj, $maxDays);
		if (!$postUrl) {
			return $keyphrase;
		} else {
			if  (preg_match('~padding-right: 13px; background: url\(.*?\) center right no-repeat;~i', $this->extraHTML)) {
				$this->extraHTML = preg_replace('~padding-right: [0-9]+px; background: url\(.*?\) center right no-repeat;~i', '', $this->extraHTML);
			}
			return "<a href=\"$postUrl\" {$this->extraHTML} title=\"$description\">$keyphrase</a>";
		}
	}
	
	/**
	 * Finds posts that have the keyphrase
	 *
	 * Method searches through the database for posts that have the keyphrase in either the
	 * post title, post content, or post excerpt.  In then checks to see if the number of
	 * points for that post are greater than or equal to the sensitivity setting.  It returns
	 * the URL to the post with the highest number of points.  Retruns FALSE if no posts are
	 * found.
	 * The method searches for matching posts in two different ways.  If the relevant columns
	 * have FULLTEXT indexes, it will use MySQL's faster MATCH AGAINST query method.  If the
	 * columns don't have FULLTEXT indexes, it will use a slower PHP method to find points
	 * for each post.
	 *
	 * @param	obj	$postObj		The post object
	 * @param	int	$maxDays		The post must have been modified before this number of days
	 * @return	mixed
	 */
	function findPosts($postObj, $maxDays) {
		$thisPostID = $postObj->ID;
		
		// - There are no FULLTEXT indexes on the columns
		if (!$this->fullText) {
			// - Find posts that have the keyphrase in either the title, excerpt, or content
			$results = $this->wpdb->get_results	("
										SELECT ID, post_title, post_excerpt, post_content
										FROM {$this->wpdb->posts}
										WHERE ID != $thisPostID
										AND ((post_content LIKE '%{$this->realKeyphrase}%')
										OR (post_title LIKE '%{$this->realKeyphrase}%')
										OR (post_excerpt LIKE '%{$this->realKeyphrase}%'))
										AND post_status = 'publish'
										AND  DATE_SUB(CURDATE(),INTERVAL $maxDays DAY) <= post_modified
											", ARRAY_A);
			
			if (!$results) {
				return false;
			}
			
			/**
			 * Now loop through all the returned results.  Use PHP's substr_count() funtion to
			 * find out how many times the keyphrase is in the title, content, or excerpt.  Do
			 * simple math to apply points to each post.
			 */
			$postRel = array();
			foreach ($results as $result) {
				$postID = $result['ID'];
				$postRel[$postID] = 0;
				$postTitle = strtolower($result['post_title']);
				$postExcerpt = strtolower($result['post_excerpt']);
				$postContent = strtolower($result['post_content']);
				
				$postRel[$postID] += substr_count($postTitle, strtolower($this->realKeyphrase)) * DEEPER_BLOG_FOUND_TITLE;
				$postRel[$postID] += substr_count($postExcerpt, strtolower($this->realKeyphrase)) * DEEPER_BLOG_FOUND_EXCERPT;
				$postRel[$postID] += substr_count($postContent, strtolower($this->realKeyphrase)) * DEEPER_BLOG_FOUND_CONTENT;
				
				// - If the post doesn't meet the sensitivity setting, ditch it
				if ($postRel[$postID] < DEEPER_BLOG_SENSITIVITY) {
					unset($postRel[$postID]);
				}
			}
			
			if (empty($postRel)) {
				return false;
			}
			
			// - Sort the found posts from greatest points to least, and return the post URL with the most points
			arsort($postRel);
			$keys = array_keys($postRel);
			$postID = $keys[0];
			return get_permalink($postID);
		
		// - There ARE FULLTEXT indexes on the columns
		} else {
			$results = $this->wpdb->get_row	("
									SELECT ID, (
									(" . DEEPER_BLOG_FOUND_TITLE . " * (MATCH (`post_title`) AGAINST ('{$this->realKeyphrase}' IN BOOLEAN MODE))) +
									(" . DEEPER_BLOG_FOUND_EXCERPT . " * (MATCH (`post_excerpt`) AGAINST ('{$this->realKeyphrase}' IN BOOLEAN MODE))) +
									(" . DEEPER_BLOG_FOUND_CONTENT . " * (MATCH (`post_content`) AGAINST ('{$this->realKeyphrase}' IN BOOLEAN MODE)))
									) AS `relevance` FROM `{$this->wpdb->posts}` WHERE (
									MATCH (`post_title`, `post_excerpt`, `post_content`)
									AGAINST ('{$this->realKeyphrase}' IN BOOLEAN MODE))
									AND ID != $thisPostID
									AND post_status = 'publish'
									AND  DATE_SUB(CURDATE(),INTERVAL $maxDays DAY) <= post_modified
									HAVING `relevance` >= " . DEEPER_BLOG_SENSITIVITY . "
									ORDER BY `relevance` DESC LIMIT 1
										", ARRAY_A);
			if (!$results) {
				return false;
			}
			return get_permalink($results['ID']);
		}
	}
	
	/**
	 * Checks for FULLTEXT indexes on columns
	 *
	 * Checks if the post_title, post_excerpt, and post_content columns in the wp_posts table
	 * have FULLTEXT indexes on them.  Retuns FALSE if they don't, and TRUE If they do.  This
	 * method also sets the $fullTextChecked and $fullText properties.
	 *
	 * @return	bool
	 */
	function isFullText() {
		if ($this->fullTextChecked) {
			return $this->fullText;
		}
		$results = $this->wpdb->get_results("SHOW INDEX FROM `{$this->wpdb->posts}`", ARRAY_A);
		$titleFullText = false;
		$contentFullText = false;
		$excerptFullText = false;
		foreach ($results as $result) {
			if ($result['Column_name'] == 'post_title') {
				if (!empty($result['Index_type']) && $result['Index_type'] == 'FULLTEXT') {
					$titleFullText = true;
				}
			}
			if ($result['Column_name'] == 'post_content') {
				if (!empty($result['Index_type']) && $result['Index_type'] == 'FULLTEXT') {
					$contentFullText = true;
				}
			}
			if ($result['Column_name'] == 'post_excerpt') {
				if (!empty($result['Index_type']) && $result['Index_type'] == 'FULLTEXT') {
					$excerptFullText = true;
				}
			}
		}
		if (!$titleFullText || !$contentFullText || !$excerptFullText) {
			$this->fullTextChecked = true;
			$this->fullText = false;
			return false;
		}
		$this->fullTextChecked = true;
		$this->fullText = true;
		return true;
	}
}